//
//  RNZipArchive.m
//  RNZipArchive
//
//  Created by Perry Poon on 8/26/15.
//  Copyright (c) 2015 Perry Poon. All rights reserved.
//

#import "RNZipArchive.h"
#import "tazip.h"

#if __has_include(<React/RCTEventDispatcher.h>)
#import <React/RCTEventDispatcher.h>
#else
#import "RCTBridge.h"
#import "RCTEventDispatcher.h"
#endif

@implementation RNZipArchive

@synthesize bridge = _bridge;
@synthesize unzipProgress;
@synthesize unzippedFilePath;

RCT_EXPORT_MODULE();

RCT_EXPORT_METHOD(isPasswordProtected:(NSString *)file
                  resolver:(RCTPromiseResolveBlock)resolve
                  rejecter:(RCTPromiseRejectBlock)reject) {

    BOOL isPasswordProtected = [SSZipArchive isFilePasswordProtectedAtPath:file];
    resolve([NSNumber numberWithBool:isPasswordProtected]);
}

RCT_EXPORT_METHOD(unzip:(NSString *)from
                  destinationPath:(NSString *)destinationPath
                  charset:(NSString *)charset
                  resolver:(RCTPromiseResolveBlock)resolve
                  rejecter:(RCTPromiseRejectBlock)reject) {
    self.unzipProgress = 0.0;
    self.unzippedFilePath = @"";
    [self zipArchiveProgressEvent:0 total:1]; // force 0%

    NSError *error = nil;

    NSMutableArray *files = [SSZipArchive unzipFileAtPath:from toDestination:destinationPath overwrite:YES password:nil error:&error delegate:self];

    self.unzipProgress = 1.0;
    [self zipArchiveProgressEvent:1 total:1]; // force 100%

    if (files != nil) {
        resolve(files);
    } else {
        reject(@"unzip_error", [error localizedDescription], error);
    }
}

RCT_EXPORT_METHOD(unzipWithPassword:(NSString *)from
                  destinationPath:(NSString *)destinationPath
                  password:(NSString *)password
                  resolver:(RCTPromiseResolveBlock)resolve
                  rejecter:(RCTPromiseRejectBlock)reject) {
    self.unzipProgress = 0.0;
    self.unzippedFilePath = @"";
    [self zipArchiveProgressEvent:0 total:1]; // force 0%

    NSError *error = nil;

    NSMutableArray *files = [SSZipArchive unzipFileAtPath:from toDestination:destinationPath overwrite:YES password:password error:&error delegate:self];

    self.unzipProgress = 1.0;
    [self zipArchiveProgressEvent:1 total:1]; // force 100%

    if (files != nil) {
        resolve(files);
    } else {
        reject(@"unzip_error", @"unable to unzip", error);
    }
}

RCT_EXPORT_METHOD(zip:(NSString *)from
                  destinationPath:(NSString *)destinationPath
                  resolver:(RCTPromiseResolveBlock)resolve
                  rejecter:(RCTPromiseRejectBlock)reject) {
    self.unzipProgress = 0.0;
    self.unzippedFilePath = @"";
    [self zipArchiveProgressEvent:0 total:1]; // force 0%

    NSFileManager *fileManager = [[NSFileManager alloc] init];
    BOOL isDir;
    BOOL success;
    [fileManager fileExistsAtPath:from isDirectory:&isDir];
    if (isDir) {
        success = [SSZipArchive createZipFileAtPath:destinationPath withContentsOfDirectory:from];
    } else {
        success = [SSZipArchive createZipFileAtPath:destinationPath withFilesAtPaths:@[from]];
    }

    self.unzipProgress = 1.0;
    [self zipArchiveProgressEvent:1 total:1]; // force 100%

    if (success) {
        resolve(destinationPath);
    } else {
        NSError *error = nil;
        reject(@"zip_error", @"unable to zip", error);
    }
}

RCT_EXPORT_METHOD(zipFiles:(NSArray<NSString *> *)files
                  destinationPath:(NSString *)destinationPath
                  options:(NSDictionary *)options
                  resolver:(RCTPromiseResolveBlock)resolve
                  rejecter:(RCTPromiseRejectBlock)reject) {
    self.unzipProgress = 0.0;
    self.unzippedFilePath = @"";
    [self zipArchiveProgressEvent:0 total:1]; // force 0%

    BOOL success;
    if (options != nil && [[options objectForKey:@"compression"] boolValue] == NO) {
        success = [SSZipArchive createStoreZipFileAtPath:destinationPath withFilesAtPaths:files];
    } else {
        success = [SSZipArchive createZipFileAtPath:destinationPath withFilesAtPaths:files];
    }

    self.unzipProgress = 1.0;
    [self zipArchiveProgressEvent:1 total:1]; // force 100%

    if (success) {
        resolve(destinationPath);
    } else {
        NSError *error = nil;
        reject(@"zip_error", @"unable to zip", error);
    }
}

RCT_EXPORT_METHOD(zipWithPassword:(NSString *)from
                  destinationPath:(NSString *)destinationPath
                  password:(NSString *)password
                  encryptionType:(NSString *)encryptionType
                  resolver:(RCTPromiseResolveBlock)resolve
                  rejecter:(RCTPromiseRejectBlock)reject) {
    self.unzipProgress = 0.0;
    self.unzippedFilePath = @"";
    [self zipArchiveProgressEvent:0 total:1]; // force 0%

    NSFileManager *fileManager = [[NSFileManager alloc] init];
    BOOL isDir;
    BOOL success;
    [fileManager fileExistsAtPath:from isDirectory:&isDir];
    if (isDir) {
        success = [SSZipArchive createZipFileAtPath:destinationPath withContentsOfDirectory:from withPassword:password];
    } else {
        success = [SSZipArchive createZipFileAtPath:destinationPath withFilesAtPaths:@[from] withPassword:password];
    }

    self.unzipProgress = 1.0;
    [self zipArchiveProgressEvent:1 total:1]; // force 100%

    if (success) {
        resolve(destinationPath);
    } else {
        NSError *error = nil;
        reject(@"zip_error", @"unable to zip", error);
    }
}

RCT_EXPORT_METHOD(zipEntryToBase64:(NSDictionary *)params
                  resolver:(RCTPromiseResolveBlock)resolve
                  rejecter:(RCTPromiseRejectBlock)reject) {
    
    NSString *archive = [params objectForKey:@"archiveName"];
    NSString *file = [params objectForKey:@"fileName"];
    
    if (archive && file) {
        void *bytes = nil;
        unsigned long length = 0;
        int ret = 0;
        
        ret = tazip_extract([archive cStringUsingEncoding:NSUTF8StringEncoding], [file cStringUsingEncoding:NSUTF8StringEncoding],&bytes,&length);
        
        if (ret) {
            NSData *imageData = [[NSData alloc] initWithBytesNoCopy:bytes length:length freeWhenDone:YES];
            NSString *base64Encoded = [imageData base64EncodedStringWithOptions:0];
            resolve(base64Encoded);
        } else {
            NSError *error = nil;
            reject(@"zip_error", @"unable to unzip file", error);
        }
    } else {
        NSError *error = nil;
        reject(@"zip_error", @"unable to unzip file", error);
    }
}

- (dispatch_queue_t)methodQueue {
    return dispatch_queue_create("com.mockingbot.ReactNative.ZipArchiveQueue", DISPATCH_QUEUE_SERIAL);
}

- (void)zipArchiveProgressEvent:(unsigned long long)loaded total:(unsigned long long)total  {
    self.unzipProgress = (float)loaded / (float)total;
    [self dispatchProgessEvent:self.unzipProgress unzippedFilePath:self.unzippedFilePath];
}

- (void)zipArchiveDidUnzipFileAtIndex:(NSInteger)fileIndex totalFiles:(NSInteger)totalFiles archivePath:(NSString *)archivePath unzippedFilePath:(NSString *)unzippedFilePath {
    self.unzippedFilePath = unzippedFilePath;
    [self dispatchProgessEvent:self.unzipProgress unzippedFilePath:self.unzippedFilePath];
}

- (void)dispatchProgessEvent:(float)progress unzippedFilePath:(NSString *)unzippedFilePath {
    [self.bridge.eventDispatcher sendAppEventWithName:@"zipArchiveProgressEvent" body:@{
                                                                                        @"progress": @(progress),
                                                                                        @"filePath": unzippedFilePath
                                                                                        }];
}

@end
