#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>



#define _MAX_PATH 256

/* Offsets of values in headers */
#define LOCVER  0               /* version needed to extract */
#define LOCFLG  2               /* encrypt, deflate flags */
#define LOCHOW  4               /* compression method */
#define LOCTIM  6               /* last modified file time, DOS format */
#define LOCDAT  8               /* last modified file date, DOS format */
#define LOCCRC  10              /* uncompressed crc-32 for file */
#define LOCSIZ  14              /* compressed size in zip file */
#define LOCLEN  18              /* uncompressed size */
#define LOCNAM  22              /* length of filename */
#define LOCEXT  24              /* length of extra field */

#define CENVEM  0               /* version made by */
#define CENVER  2               /* version needed to extract */
#define CENFLG  4               /* encrypt, deflate flags */
#define CENHOW  6               /* compression method */
#define CENTIM  8               /* last modified file time, DOS format */
#define CENDAT  10              /* last modified file date, DOS format */
#define CENCRC  12              /* uncompressed crc-32 for file */
#define CENSIZ  16              /* compressed size in zip file */
#define CENLEN  20              /* uncompressed size */
#define CENNAM  24              /* length of filename */
#define CENEXT  26              /* length of extra field */
#define CENCOM  28              /* file comment length */
#define CENDSK  30              /* disk number start */
#define CENATT  32              /* internal file attributes */
#define CENATX  34              /* external file attributes */
#define CENOFF  38              /* relative offset of local header */

#define EXTCRC  0               /* uncompressed crc-32 for file */
#define EXTSIZ  4               /* compressed size in zip file */
#define EXTLEN  8               /* uncompressed size */

#define ENDDSK  0               /* number of this disk */
#define ENDBEG  2               /* number of the starting disk */
#define ENDSUB  4               /* entries on this disk */
#define ENDTOT  6               /* total number of entries */
#define ENDSIZ  8               /* size of entire central directory */
#define ENDOFF  12              /* offset of central on starting disk */
#define ENDCOM  16              /* length of zip file comment */


static int tazip_findFileOffset(FILE *fd,char *toSearch, unsigned long *offset,unsigned long *len);
static int findZipFile(long startOfRecord,char *toSearch, unsigned long *offset,unsigned long *len, FILE *fd);
static void getZipFileOffset(FILE *fd, unsigned long startOfLocal,unsigned long *offset,unsigned long *len);

/*
 int main(void){
 
 void *b;
 int a;
 tazip_extract("page.zip", "pag_0_0.jpg", &b, &a);
 }
 */

int tazip_extract(char *zipname, char *relativeName, void **buffer, unsigned long *bufferLen)
{
	int toret;
	unsigned long fileOffset, fileLen;
	int ret;
	FILE *fd;
	
	toret = 0;
	*buffer = NULL;
	
	if (zipname && relativeName && strlen(relativeName)>0) {
		fd = fopen(zipname, "rb");
		if (fd) {
			ret = tazip_findFileOffset(fd,relativeName,&fileOffset,&fileLen);
			if (ret == 1) {
				
				*buffer = (void *)malloc(fileLen);
				ret = fseek(fd,fileOffset,SEEK_SET);
				if(ret == 0){
					ret = fread(*buffer,1,fileLen,fd);
					if(ret == fileLen){
						toret = 1;
						*bufferLen = fileLen;
					}
				}
			}
			
			fclose(fd);
		}
	}         
	
	if(!toret){
		*bufferLen = 0;
		if(*buffer){
			free(*buffer);
			*buffer = NULL;
		}
	}
	
	return toret;
}


static int tazip_findFileOffset(FILE *fd,char *toSearch, unsigned long *offset,unsigned long *len) {
	long ret;
	unsigned long startOfTable;
	int toret=0;
	int fLength;
	unsigned char byteVal[4];
	
	
	// Calculate the whole filesize
	fLength = fseek(fd,0L,SEEK_END);
	if (fLength == 0) {      
		ret = fseek(fd,-6L,SEEK_END);
		if(ret == 0){
			startOfTable = 0;
			ret = fread(byteVal,1,4,fd);
			startOfTable += (unsigned long)(byteVal[3] << 24L);
			startOfTable += (unsigned long)(byteVal[2] << 16L);
			startOfTable += (unsigned long)(byteVal[1] << 8L);
			startOfTable += (unsigned long)(byteVal[0]);
			if (ret == 4L && startOfTable) {
				toret = findZipFile(startOfTable,toSearch,offset,len,fd);      
			}
		}
		// codice aggiunto per pararsi se alla fine del file viene aggiunto in CRLF in piu,
		// in questo caso l'offset per andare a leggere il central header e' sbagliato e bisogna
		// rileggerlo senza i CRLF
		if (toret == 3) {
			ret = fseek(fd,-8L,SEEK_END);
			if(ret == 0){
				startOfTable = 0;
				ret = fread(byteVal,1,4,fd);
				startOfTable += (unsigned long)(byteVal[3] << 24L);
				startOfTable += (unsigned long)(byteVal[2] << 16L);
				startOfTable += (unsigned long)(byteVal[1] << 8L);
				startOfTable += (unsigned long)(byteVal[0]);
				if (ret == 4L && startOfTable) {
					toret = findZipFile(startOfTable,toSearch,offset,len,fd);
				}
			}
		}
	}
	return toret;
}

static int findZipFile(long startOfRecord,char *toSearch, unsigned long *offset,unsigned long *length, FILE *fd) {
	int len, fileNameLen;
	long ret, localOffset;
	char buffer[50];
	char zipFileName[_MAX_PATH];
	int toret = 0;
	unsigned char *pos;
	long tmpoffset = 0;
	
	//go to start of descriptor
	ret = fseek(fd,startOfRecord,SEEK_SET);
	
	while (!toret && ret == 0 && fread(buffer,1,46,fd)) {      
		if (buffer[0] == 'P' && buffer[1] == 'K' ) {
			//read filename length header
			fileNameLen = (int)(buffer[CENNAM+4]);
			if (fileNameLen > 0 && fileNameLen <= _MAX_PATH-1) {
				//get filename ( eventually with path )
				len = fread(zipFileName,1,fileNameLen, fd);
				if (len == fileNameLen) {
					zipFileName[fileNameLen] = 0;
					if (strcasecmp(toSearch,zipFileName)==0) {
						//file found, get informationm about it
						
						//memcpy(&localOffset,buffer+CENOFF+4,sizeof(long));
						pos = (unsigned char *)(buffer+CENOFF+4);
						localOffset = (unsigned long)(pos[3] << 24L);
						localOffset += (unsigned long)(pos[2] << 16L);
						localOffset += (unsigned long)(pos[1] << 8L);
						localOffset += (unsigned long)(pos[0]);
						
						getZipFileOffset(fd,localOffset,offset,length);
						
						if(*length == 0){
							/* get len from end central dir */
							pos   = (unsigned char *)(buffer+LOCLEN + 2);   
							*length  = (long)(pos[3] << 24L);
							*length += (long)(pos[2] << 16L);
							*length += (long)(pos[1] << 8L);
							*length += (long)(pos[0]);                     
						}
						
						toret = 1;
					} else {
						//go to next central header
						//tmpoffset += 46 + len;
						//tmpoffset += (long)(buffer[CENEXT+4]) + (int)(buffer[CENCOM+4]);
						//ret = fseek(fd, startOfRecord + tmpoffset, SEEK_SET);
						
						tmpoffset = (long)(buffer[CENEXT+4]) + (int)(buffer[CENCOM+4]);
						ret = fseek(fd, tmpoffset , SEEK_CUR);
						
						
					}
				} else {
					toret = 2;
				}
			} else {
				toret = 2;
			}
		} else {
			toret = 3;
		}
	}
	// 0 Not found, 1 Found, 2 Error
	return toret;
}

static void getZipFileOffset(FILE *fd, unsigned long startOfLocal,unsigned long *offset,unsigned long *len) {
	long ret;
	char buffer[30];
	unsigned char *pos;
	
	ret = fseek(fd,startOfLocal+4,SEEK_SET);
	if (ret == 0 && fread(&buffer,1,26,fd) ) {
		// localoffset + 4 + lenLocalHeader + lenFileName + lenExtraInfo
		
		//*offset = startOfLocal + 4 + 26 + (long)(*(short *)(buffer+LOCNAM)) + (long)(*(short *)(buffer+LOCEXT));
		*offset  = startOfLocal + 4 + 26;
		pos      = (unsigned char *)(buffer+LOCNAM);
        
        //fix for archive.zipgraph extra bytes
        long locnameLen = (long)(pos[1] << 8L) + (long)(pos[0]);
        ret = fseek(fd,startOfLocal+4+26+locnameLen,SEEK_SET);
        char extra5char=0;
        if (ret == 0 && fread(&extra5char,1,1,fd) ) {
            if (extra5char == 1){
                *offset +=5;
            }
        }
		*offset += (long)(pos[1] << 8L);
		*offset += (long)(pos[0]);
		pos      = (unsigned char *)(buffer+LOCEXT);
		*offset += (long)(pos[1] << 8L);
		*offset += (long)(pos[0]);      
		
		//*len = *((long *)(buffer+LOCLEN));
		pos   = (unsigned char *)(buffer+LOCLEN);   
		*len  = (long)(pos[3] << 24L);
		*len += (long)(pos[2] << 16L);
		*len += (long)(pos[1] << 8L);
		*len += (long)(pos[0]);      
	}
}
